/*
 * Mòdul WiFly driver, comunica amb un wifly RN170 a través de les UARTS de la placa LPC1769
 *
 *  Created on: 23/10/2015
 *      Author: Victor
 *  revision 1.0 date 23/10/2015
 */

#ifndef __WIFLY_H_
#define __WIFLY_H_

/* WIFLY MODE */
#define DATA_M	0	///< Wifly data mode */
#define CMD_M	1	///< Wifly command mode */

/* RESET MODE */
#define RESET_HARDWARE 0 ///< Hardware reset mode */
#define RESET_SOFTWARE 1 ///< Software reset mode */

/* WIFLY CONNECTION STATUS */
#define WLAN_DISCON	0		///< Wlan disconnected
#define WLAN_CON	1		///< Wlan connected
#define WLAN_APCON	2		///< AP connected

/* COMUNICATION MODE */
#define MODE_DISCON	0	///< Protocol not configured
#define MODE_UDP	1	///< UDP Protocol configured
#define MODE_TCP	2	///< TCP Protocol configured
#define MODE_HTTP	3	///< HTTP Protocol configured


/* BUFFER SIZE */
#define WIFLYBUFSIZE (BUFSIZE_WIFLY) /**< Wifly buffer size */

/* RESET PIN WIFLY */
#define RST_WIFLY_PIN (WIFLY_RST_PIN)

/* WLAN AUTH MODES */
#define OPEN	0	///< Open
#define WEP		1	///< WEP-128
#define WPA1	2	///< WPA1
#define MIXED	3	///< Mixed WPA1 and WPA2-SK
#define WPA2	4	///< WPA2-PSK
#define ADHOCM	6	///< Ad hoc Mode
#define WPE64	8	///< WPE-64

#define UART_MD 0x01 //ECHO OFF

/*DELAY TIME, ms*/

#define CMD_DELAY 	600
#define RST_DELAY 	3000
#define JOIN_DELAY 	8000
#define SET_DELAY 	700
#define ECHO_DELAY 	1000
#define PING_DELAY 	4000
#define GET_DELAY 	700
#define OPEN_DELAY 	6000
#define UDP_DELAY	3000
#define TCP_DELAY	2000


/* EXPECTED RESULTS */
#define READY "*READY*"
#define CMD "CMD"
#define EXIT "EXIT"
#define AOK "AOK"
#define JOIN_OK "Associated!"
#define PING_OK "reply"
#define OPEN_CON "*OPEN*"
#define HELLO_CON "*HELLO*"
#define CLOSE_CON "*CLOS*"
#define OK "OK"
#define HTTP_HEAD "HTTP/1.0"
#define SAVE_OK "Storing in config"


/** Wifly start Printf module and connection parameters
 *
 * @param portNum UART id
 * @param baudrate UART baudrate
 *
 * @return Returns the result of wifly initialisation. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_start( uint32_t portNum, uint32_t baudrate );

/** Wifly start Log module to send log information
 *
 * @param PortNum
 * @param baudrate	UART port baud rate.
 * @param logSeverity
 *
 * @return Returns the result of log initialisation. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_logStart( uint32_t PortNum, uint32_t baudrate, uint32_t logSeverity );

/** Change the UART and wifly baud rate communication
 *
 * @param baudrate	UART port baud rate.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_ChangeBaudRate(uint32_t baudrate);

/** Save the configuration settings.
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_save();

/** factory Reset
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_factoryReset();

/** Puts the module to sleep mode
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_sleep();

/** Reset wifly device to ensure to start from know state
 *
 * @param typeReset Reset type: HARDWARE=0, SOFTWARE=1
 */
void Wifly_reset(uint8_t typeReset);

/** Connect to WLAN ssid
 *
 * @param AuthMode WLAN auth mode
 * @param ssid Name of WLAN connection
 * @param passw WLAN Password
 *
 * @return Returns whether the connection was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_WLANConnect(uint32_t AuthMode, char *ssid, char* passw);

/** Connect to WLAN ssid and change IP Ping used to check connection
 *
 * @param AuthMode WLAN auth mode
 * @param ssid Name of WLAN connection
 * @param passw WLAN Password
 * @param pingIP Ping address
 *
 * @return Returns whether the connection was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_WLANConnectPing(uint32_t AuthMode, char *ssid, char* passw, char* pingIP);

/** Leave a join connection
 *
 * @return Returns whether the connection was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_leave();

/** Sets the WiFly module's IP address.
 *
 * @param address	IP address
 * @return Returns	whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setIpAddress(char * address);

/** Sets the WiFly module's IP address
 *
 * @param address	IP address
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setIpBackup(char* address);

/** Enables/disables DHCP mode.
 *
 * @param value Disabled=0, enabled=1
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setIpDHCP(uint8_t value);

/** Sets the network mask.
 *
 * @param address IP subnet address.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setIpNET(char* address);

/** Sets the TCP/IP functions
 *
 * @param mask is a hex number referring to a bit-mapped register. see Figure 2-1 Wifly manual.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setIpFlags(uint32_t mask);

/** Sets the gateway IP address
 *
 * @param address IP gateway address.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setIpGateway(char* address);

/** Sets the remote host's IP address
 *
 * @param address remote host's IP address
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setIpHost(char* address);

/** Sets the local port number
 *
 * @param value is a decimal number representing the port.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setIpLocalPort(uint32_t value);

/** Sets the network mask.
 *
 * @param address
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setIpnetmask(char* address);

/** Sets the IP protocol.
 *
 * @param value is a bit-mapped register. UDP=0, TCP=2, HTTP... See Figure 2-2 Wifly manual.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setIpProtocol(uint32_t value);

/** Sets the remote host port number.
 *
 * @param value port number
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setIpRemote(uint32_t value);

/** Controls the TCP connect timers, DNSpreferences, and remote configuration options.
 *
 * @param value
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setIpTcpMode(uint32_t value);

/** Sets the ASCI string that is sent to the remote TCP client when the TCP port is opened.
 *
 * @param value String to send to the remote TCP client.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setComRemote(char* value);

/** Sets the HTTP client/web server information.
 *
 * @param value is a bitmapped register as shown in Figure 2-4 of wifly manual.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setOptionFormat(uint32_t value);

/** Sets Uart mode.
 *
 * @param value is a hex number masking a bit-mapped value as shown in Figure 2-7 of wifly manual.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setUartMode(uint32_t value);

/** Immediately changes the baud rate
 *
 * @param value is 2400, 4800, 9600, 19200, 38400, 57600, 115200, 230400, 460800, or 921600
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setUartInstant(uint32_t value);

/** Change the baud rate
 *
 * @param value is 2400, 4800, 9600, 19200, 38400, 57600, 115200, 230400, 460800, or 921600
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setUartBaud(uint32_t value);

/** Set wlan tx command to set wlan tx power level
 *
 * @param tx tx power value (dBm)
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setWlanTX(uint32_t tx);

/** Sets the authentication mode.
 *
 * @param value is a decimal value as shown in Figure 2-13 Wifly manual
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setWlanAuth(uint32_t value);

/** Sets the WLAN channel.
 *
 * @param value is a decimal number from 1 to 13 representing a fixed channel
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setWlanChannel(uint8_t value);

/** Sets the policy for automatically associating with network access points,where <value> is one of the options.
 *
 * @param value 0= Manual, 1= Associate with the access point that matches the stored SSID, 2= Associated with any, 4=
 * Create an ad hoc network, 5=Associaed an Soft AP.
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setWlanJoin(uint32_t value);

/** sets the WEP-64 or WEP-128 key
 *
 * @param password is 26 ASCII chars in hex without the preceding 0x.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setWlanKey(char* password);

/** sets the passphrase for WPA and WPA2 security modes
 *
 * @param password is an alphanumeric pass phrase.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setWlanPhrase(char* password);

/** sets the SSID with which the module associates.
 *
 * @param value is 1 to 32 characters that represents SSID name.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setWlanSSID(char* value);

/** sets the name of the host for TCP/IP connections
 *
 * @param value DNS Name max 32 charecters.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_setDNSName(char* value);

/** Ping an IP address
 *
 * @param ip IP address
 * @return Returns whether the ping was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_ping(char *ip);

/** Get ip command to known IP address.
 *
 * @return IP address.
 */
char * Wifly_getIp();

/** Get wlan command to known wlan channel.
 *
 * @return Channel number.
 */
uint8_t Wifly_getChannel();

/** Get wlan tx command to known wlan tx power level.
 *
 * @return tx power.
 */
uint8_t Wifly_getTx();

/** Get Mac Address.
 *
 * @return MAC Address.
 */
char* Wifly_getMac();

/** Get SSID connection name.
 *
 * @return SSID
 */
char* Wifly_getWlan();

/** Get Wifly version
 *
 * @return Wifly firmware version
 */
double Wifly_getVer();


/** Show rssi command to known rssi signal.
 *
 * @return rssi signal (dBm).
 */
int32_t Wifly_getRSSI();

/** Show time up command response.Return time in seconds since wifly start up.
 *
 * @return time since last start up (s).
 */
uint32_t Wifly_getTime();

/** Sending the command settings necessary to establish WLAN connection.
 *
 * @param AuthMode is a decimal that represents the authentication mode.
 * @param ssid SSID name.
 * @param passw password of the SSID connection.
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_WLANConfigConnect(uint32_t AuthMode, char *ssid, char* passw);

/** Send a HTTP command and receive the server response.
 *
 * @param dnsAddress web page DNS address.
 * @param getCommand get command string format.
 * @param port Web server port connection.
 * @param proto Ip protocol configuration number.
 *
 * @return string response.
 */
char * Wifly_HTTPReceive(char *dnsAddress, char *getCommand, uint32_t port, uint8_t proto);

/** Send a HTTP command to send values.
 *
 * @param dnsAddress web page DNS address.
 * @param getCommand get command string format.
 * @param port Web server port connection.
 * @param proto Ip protocol configuration number.
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_HTTPSend(char *dnsAddress, char *getCommand, uint32_t port, uint8_t proto);

/** Sending the command settings necessary to HTTP connection.
 *
 * @param dnsAddress web page DNS address.
 * @param getCommand get command string format.
 * @param port Web server port connection.
 * @param proto Ip protocol configuration number.
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_HTTPConfig(char *dnsAddress, char *getCommand, uint32_t port, uint8_t proto);

/** Send a UDP command and receive the server response.
 *
 * @param address remote IP address.
 * @param localPort local port number.
 * @param remotePort remote port number connection.
 * @param command Command string to send.
 *
 * @return String response.
 */
char * Wifly_UDPReceive(char *address, uint32_t localPort, uint32_t remotePort, const char * restrict command,...);

/** Send a UDP command to send values.
 *
 * @param address Remote IP address.
 * @param localPort Local port number.
 * @param remotePort Remote port number connection.
 * @param exResp Expected string response.
 * @param command Command string to send.
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_UDPSend(char *address, uint32_t localPort, uint32_t remotePort, char *exResp , const char * restrict command,...);

/** Sending the command settings necessary to UDP connection
 *
 * @param address Remote IP address.
 * @param localPort Local port number.
 * @param remotePort Remote port number connection.
 *
 * @return Returns whether the configuration was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_UDPConfig(char *address, uint32_t localPort, uint32_t remotePort);

/** Send a TCP command and receive the server response.
 *
 * @param address remote IP address.
 * @param remotePort remote port number connection.
 * @param command string to send.
 *
 * @return String response.
 */
char * Wifly_TCPReceive(char *address, uint32_t remotePort, const char * restrict command,...);

/** Send a TCP command to send values.
 *
 * @param address IP address.
 * @param remotePort remote port number.
 * @param exResp expected string response.
 * @param command string to send.
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_TCPSend(char *address, uint32_t remotePort, char *exResp , const char * restrict command,...);

/** Send every time interval a TCP command to send values.
 *
 * /attention To disable Wifly_TCPTimeOpen you should to call Wifly_TCPTimeOpenDisc in a different task.
 *
 * @param time time interval in seconds
 * @param address IP address.
 * @param remotePort remote port number.
 * @param exResp expected string response.
 * @param command string to send.
 */
void Wifly_TCPTimeOpen(uint32_t time, char *address, uint32_t remotePort, char *exResp , const char * restrict command,...);

/** Disable TCPTimeOpen
 *
 */
void Wifly_TCPTimeOpenDisc();

/** Sending the command settings necessary to UDP connection.
 *
 * @param address destination IP address.
 * @param port remote port number.
 *
 * @return Returns whether the configuration was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_TCPConfig(char *address, uint32_t port);

/** Get the connection status in the hex format 8(XYZ).
 *
 * @return Return 8(XYZ) is a bit-mapped register providing the information shown in Figure 2-8 of the wifly manual.
 */
uint32_t Wifly_showConnection();

/** Open a TCP connection.
 *
 * @param dnsAddress destination DNS or IP address.
 * @param port remote port number.
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_open(char* dnsAddress, uint32_t port);

/** Close a TCP connection.
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 */
uint8_t Wifly_close();

/** Creates a soft AP network on channel.
 *
 * @param ssid SSID name.
 * @param channel Connection channel number.
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 * /Caution Only use with firmware 2.45 or higher
 */
uint8_t Wifly_APMode(char *ssid, uint8_t channel);

/** Sending the command settings necessary to AP mode.
 *
 * @param ssid SSID name.
 * @param channel Connection channel number.
 * @param ipAddress Local IP address
 * @param subnetMask Local IP subnet mask.
 * @param gateway Local gateway address.
 *
 * @return Returns whether the command was successful. SUCCESS = TRUE, ERROR = FALSE.
 * /Caution Only use with firmware 2.45 or higher
 */
uint8_t Wifly_ConfigAPMode(char *ssid, uint8_t channel,char *ipAddress, char *subnetMask, char *gateway);


#endif /* WIFLY_H_ */
