/*
    FreeRTOS V8.0.1 - Copyright (C) 2014 Real Time Engineers Ltd.

    This file is part of the FreeRTOS distribution.

    FreeRTOS is free software; you can redistribute it and/or modify it under
    the terms of the GNU General Public License (version 2) as published by the
    Free Software Foundation AND MODIFIED BY the FreeRTOS exception.
    ***NOTE*** The exception to the GPL is included to allow you to distribute
    a combined work that includes FreeRTOS without being obliged to provide the
    source code for proprietary components outside of the FreeRTOS kernel.
    FreeRTOS is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details. You should have received a copy of the GNU General Public 
    License and the FreeRTOS license exception along with FreeRTOS; if not it 
    can be viewed here: http://www.freertos.org/a00114.html and also obtained 
    by writing to Richard Barry, contact details for whom are available on the
    FreeRTOS WEB site.

    1 tab == 4 spaces!

    http://www.FreeRTOS.org - Documentation, latest information, license and
    contact details.

    http://www.SafeRTOS.com - A version that is certified for use in safety
    critical systems.

    http://www.OpenRTOS.com - Commercial support, development, porting,
    licensing and training services.
*/

/* FreeRTOS.org includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "semphr.h"

/* Demo includes. */
#include "basic_io.h"

#include "type.h"
#include "Log.h"
#include "WiFly.h"
#include "Udp.h"
#include "Arpalab.h"
#include "stdlib.h"
#include <string.h>

#include "Config.h"

#define mainDELAY_LOOP_Producer (15000)
#define mainDELAY_LOOP_Consume 	(15000)
#define mainDELAY_WAIT_QUEUE 	(15000)
#define QUEUE_BUZSIZE 5

#define UART_FOR_PRINT UART3PORT
#define UART_FOR_DEBUG UART0PORT
#define BAUD_PRINT DEFBAUDRATE
#define BAUD_DEBUG DEFBAUDRATE

#define PING_IP			(WLAN_PING_CHECK)
#define AUTH_MODE 		(WLAN_SEC_TYPE)
#define SSID			(WLAN_SSID)		//"MORGUE"
#define PASSWORD 		(WLAN_PASSWORD)

/* Arpalab settings */
#define ARPA_ID 		(ARPALAB_ID)
#define ARPA_APP 		(ARPALAB_APP)
#define DNS_ADDRESS 	(ARPALAB_DNSADDRESS)

/* TCP settings */

#define TCP_SRV_ADDRESS		(TCP_ADDRESS)
#define TCP_SRV_PORT 		(TCP_PORT)

/* UDP settings */
#define UDP_SRV_ADDRESS		(UDP_ADDRESS)
#define UDP_SRV_PORT 		(UDP_PORT)
#define UDP_SRV_LOCAL_PORT 		(UDP_LOCAL_PORT)

/* The tasks to be created.  Two instances are created of the Producer task while
only a single instance is created of the Consumer task. */
static void vProducerHTTPTask( void *pvParameters );
static void vConsumerHTTPTask( void *pvParameters );
static void vProducerTCPTask( void *pvParameters );
static void vConsumerTCPTask( void *pvParameters );
static void vProducerUDPTask( void *pvParameters );
static void vConsumerUDPTask( void *pvParameters );

/*-----------------------------------------------------------*/

/* Declare a variable of type xQueueHandle.  This is used to store the queue
that is accessed by all three tasks. */
xQueueHandle xHTTPQueue, xTCPQueue, xUDPQueue;


int main( void )
{
    /* The queue is created to hold a maximum of 5 long values. */
    xHTTPQueue = xQueueCreate( QUEUE_BUZSIZE, sizeof( uint32_t ) );
    xTCPQueue = xQueueCreate( QUEUE_BUZSIZE, sizeof( uint32_t ) );
    xUDPQueue = xQueueCreate( QUEUE_BUZSIZE, sizeof( uint32_t ) );

    /* Initialize Log and Wifly module */
	Wifly_start( UART_FOR_PRINT, BAUD_PRINT );
	Wifly_logStart(UART_FOR_DEBUG, BAUD_DEBUG,LOG_LVL);
    Arpalab_start(SSID, AUTH_MODE, PASSWORD);
    Tcp_start(SSID, AUTH_MODE, PASSWORD);
    Udp_start(SSID, AUTH_MODE, PASSWORD);

	if( (xHTTPQueue != NULL) && (xTCPQueue != NULL) && (xUDPQueue != NULL) )
	{
		/* Create the task that will write the received value from Arp@lab */
		xTaskCreate( vProducerHTTPTask, "HTTP Producer", 240, NULL , 1, NULL );
		/* Create the task that will read from the queue for send it to Arp@lab  */
		xTaskCreate( vConsumerHTTPTask, "HTTP Consumer", 240, NULL, 2, NULL );

		/* Create the task that will write the received value from TCP server */
		xTaskCreate( vProducerTCPTask, "TCP Producer", 240, NULL , 1, NULL );
		/* Create the task that will read from the queue for send it to TCP Server  */
		xTaskCreate( vConsumerTCPTask, "TCP Consumer", 240, NULL, 2, NULL );

		/* Create the task that will write the received value from UDP server */
		xTaskCreate( vProducerUDPTask, "UDP Producer", 240, NULL , 1, NULL );
		/* Create the task that will read from the queue for send it to UDP Server  */
		xTaskCreate( vConsumerUDPTask, "UDP Consumer", 240, NULL, 2, NULL );

		/* Start the scheduler so the created tasks start executing. */
		vTaskStartScheduler();
	}
	else
	{
		/* The queue could not be created. */
		Log_log(3, 1,"The queues could not be created.");
	}
		
    /* If all is well we will never reach here as the scheduler will now be
    running the tasks.  If we do reach here then it is likely that there was 
    insufficient heap memory available for a resource to be created. */
	for( ;; );
	return 0;
}
/*-----------------------------------------------------------*/

static void vProducerHTTPTask( void *pvParameters )
{
uint32_t lValueToSend;
char string[64];
portBASE_TYPE xStatus;
const char *pcTaskName = "[HTTP] Task Producer random Arpalab running";
const portTickType xTicksToWait = 500 / portTICK_RATE_MS;


	/* As per most tasks, this task is implemented within an infinite loop. */
	for( ;; )
	{
		Log_log(LOG_LVL, LV_INFO,(char*)pcTaskName);
		lValueToSend = Arpalab_receiveValue(DNS_ADDRESS);
		/* The first parameter is the queue to which data is being sent.  The 
		queue was created before the scheduler was started, so before this task
		started to execute.

		The second parameter is the address of the data to be sent.

		The third parameter is the Block time � the time the task should be kept
		in the Blocked state to wait for space to become available on the queue
		should the queue already be full.  In this case we don�t specify a block
		time because there should always be space in the queue. */
		while( uxQueueMessagesWaiting( xHTTPQueue ) == QUEUE_BUZSIZE )
		{
			vTaskDelay(mainDELAY_WAIT_QUEUE);
		}
		xStatus = xQueueSendToBack( xHTTPQueue, &lValueToSend, xTicksToWait );
		Log_log(LOG_LVL, LV_INFO,"[HTTP]ProducerHTTP: value received from the server= %d",lValueToSend );

		if( xStatus != pdPASS )
		{
			/* We could not write to the queue because it was full � this must
			be an error as the queue should never contain more than one item! */
			Log_log(LOG_LVL, LV_ERR,"[HTTP] The value could not write to the queue." );

		}

		/* Allow the other Producer task to execute. */
		taskYIELD();
		vTaskDelay( mainDELAY_LOOP_Producer);
	}
}
/*-----------------------------------------------------------*/
static void vConsumerHTTPTask( void *pvParameters )
{
/* Declare the variable that will hold the values received from the queue. */
uint32_t lReceivedValue;
portBASE_TYPE xStatus;
char string[64];
const char *pcTaskName = "[HTTP] Task Consumer Arpalab is running";
const portTickType xTicksToWait = 500 / portTICK_RATE_MS;

	/* This task is also defined within an infinite loop. */
	for( ;; )
	{
		vTaskDelay( mainDELAY_LOOP_Consume);
		Log_log(LOG_LVL, LV_INFO,(char*)pcTaskName);
		/* Log when there are full queue */
		if( uxQueueMessagesWaiting( xHTTPQueue ) == QUEUE_BUZSIZE )
		{
			Log_log(LOG_LVL, LV_ERR, "[HTTP] The queue is full!" );
		}

		/* The first parameter is the queue from which data is to be received.  The
		queue is created before the scheduler is started, and therefore before this
		task runs for the first time.

		The second parameter is the buffer into which the received data will be
		placed.  In this case the buffer is simply the address of a variable that
		has the required size to hold the received data. 

		the last parameter is the block time � the maximum amount of time that the
		task should remain in the Blocked state to wait for data to be available should
		the queue already be empty. */
		xStatus = xQueueReceive( xHTTPQueue, &lReceivedValue, xTicksToWait );


		if( xStatus == pdPASS )
		{
			/* Data was successfully received from the queue, print out the received
			value. */
			while (Arpalab_sendValue( DNS_ADDRESS,ARPA_ID, ARPA_APP, lReceivedValue)==FALSE)
			{
				Log_log(LOG_LVL, LV_ERR,"[HTTP] Cannot send value to Arpalab");
			}
			Log_log(LOG_LVL, LV_INFO, "[HTTP] Consumer HTTP: Sended value to Arpalab= %d",lReceivedValue );
		}
		else
		{
			/* We did not receive anything from the queue even after waiting for 100ms.
			This must be an error as the sending tasks are free running and will be
			continuously writing to the queue. */

			Log_log(LOG_LVL, LV_ERR,"[HTTP] Don't receive anything from the queue");
		}
	}
}
/*-----------------------------------------------------------*/
static void vProducerTCPTask( void *pvParameters )
{
uint32_t lValueToSend;
char string[64];
portBASE_TYPE xStatus;
const char *pcTaskName = "[TCP] Task Producer random TCP server running";
const portTickType xTicksToWait = 500 / portTICK_RATE_MS;


	/* As per most tasks, this task is implemented within an infinite loop. */
	for( ;; )
	{
		Log_log(LOG_LVL, LV_INFO,(char*)pcTaskName);
		lValueToSend = Tcp_receiveValue(TCP_SRV_ADDRESS, TCP_SRV_PORT);

		while( uxQueueMessagesWaiting( xTCPQueue ) == QUEUE_BUZSIZE )
		{
			vTaskDelay(mainDELAY_WAIT_QUEUE);
		}
		/* The first parameter is the queue to which data is being sent.  The
		queue was created before the scheduler was started, so before this task
		started to execute.

		The second parameter is the address of the data to be sent.

		The third parameter is the Block time � the time the task should be kept
		in the Blocked state to wait for space to become available on the queue
		should the queue already be full.  In this case we don�t specify a block
		time because there should always be space in the queue. */
		xStatus = xQueueSendToBack( xTCPQueue, &lValueToSend, xTicksToWait );
		Log_log(LOG_LVL, LV_INFO,"[TCP] ProducerTCP: value received from the server= %d",lValueToSend );

		if( xStatus != pdPASS )
		{
			/* We could not write to the queue because it was full � this must
			be an error as the queue should never contain more than one item! */
			Log_log(LOG_LVL, LV_ERR,"[TCP] ProducerTCP: The value could not write to the queue." );

		}

		/* Allow the other Producer task to execute. */
		taskYIELD();
		vTaskDelay( mainDELAY_LOOP_Producer);
	}
}
/*-----------------------------------------------------------*/
static void vConsumerTCPTask( void *pvParameters )
{
/* Declare the variable that will hold the values received from the queue. */
uint32_t lReceivedValue;
portBASE_TYPE xStatus;
char string[64];
const char *pcTaskName = "[TCP] Task Consumer TCP server is running";
const portTickType xTicksToWait = 500 / portTICK_RATE_MS;

	/* This task is also defined within an infinite loop. */
	for( ;; )
	{
		vTaskDelay( mainDELAY_LOOP_Consume);
		Log_log(LOG_LVL, LV_INFO,(char*)pcTaskName);
		/* Log when there are full queue */
		if( uxQueueMessagesWaiting( xTCPQueue ) == QUEUE_BUZSIZE )
		{
			Log_log(LOG_LVL, LV_ERR, "[TCP] The queue is full!" );
		}

		/* The first parameter is the queue from which data is to be received.  The
		queue is created before the scheduler is started, and therefore before this
		task runs for the first time.

		The second parameter is the buffer into which the received data will be
		placed.  In this case the buffer is simply the address of a variable that
		has the required size to hold the received data.

		the last parameter is the block time � the maximum amount of time that the
		task should remain in the Blocked state to wait for data to be available should
		the queue already be empty. */
		xStatus = xQueueReceive( xTCPQueue, &lReceivedValue, xTicksToWait );


		if( xStatus == pdPASS )
		{
			/* Data was successfully received from the queue, print out the received
			value. */
			while (Tcp_sendValue(TCP_SRV_ADDRESS, TCP_SRV_PORT, lReceivedValue)==FALSE)
			{
				Log_log(LOG_LVL, LV_ERR,"[TCP] Cannot send value to TCP server");
			}
			Log_log(LOG_LVL, LV_INFO, "[TCP] Consumer TCP: Sended value to TCP server= %d",lReceivedValue );
		}
		else
		{
			/* We did not receive anything from the queue even after waiting for 100ms.
			This must be an error as the sending tasks are free running and will be
			continuously writing to the queue. */

			Log_log(LOG_LVL, LV_ERR,"[TCP] Don't receive anything from the queue");
		}

	}
}
/*-----------------------------------------------------------*/
static void vProducerUDPTask( void *pvParameters )
{
uint32_t lValueToSend;
char string[64];
portBASE_TYPE xStatus;
const char *pcTaskName = "[UDP] Task Producer random UDP server running";
const portTickType xTicksToWait = 500 / portTICK_RATE_MS;


	/* As per most tasks, this task is implemented within an infinite loop. */
	for( ;; )
	{
		Log_log(LOG_LVL, LV_INFO,(char*)pcTaskName);
		lValueToSend = Udp_receiveValue(UDP_SRV_ADDRESS, UDP_SRV_LOCAL_PORT, UDP_SRV_PORT);

		while( uxQueueMessagesWaiting( xUDPQueue ) == QUEUE_BUZSIZE )
		{
			vTaskDelay(mainDELAY_WAIT_QUEUE);
		}
		/* The first parameter is the queue to which data is being sent.  The
		queue was created before the scheduler was started, so before this task
		started to execute.

		The second parameter is the address of the data to be sent.

		The third parameter is the Block time � the time the task should be kept
		in the Blocked state to wait for space to become available on the queue
		should the queue already be full.  In this case we don�t specify a block
		time because there should always be space in the queue. */
		xStatus = xQueueSendToBack( xUDPQueue, &lValueToSend, xTicksToWait );

		Log_log(LOG_LVL, LV_INFO,"[UDP] ProducerUDP: value received from the server= %d",lValueToSend );

		if( xStatus != pdPASS )
		{
			/* We could not write to the queue because it was full � this must
			be an error as the queue should never contain more than one item! */
			Log_log(LOG_LVL, LV_ERR,"[UDP] ProducerUDP: The value could not write to the queue." );

		}

		/* Allow the other Producer task to execute. */
		taskYIELD();
		vTaskDelay( mainDELAY_LOOP_Producer);
	}
}
/*-----------------------------------------------------------*/
static void vConsumerUDPTask( void *pvParameters )
{
/* Declare the variable that will hold the values received from the queue. */
uint32_t lReceivedValue;
portBASE_TYPE xStatus;
char string[64];
const char *pcTaskName = "[UDP] Task Consumer UDP server is running";
const portTickType xTicksToWait = 500 / portTICK_RATE_MS;

	/* This task is also defined within an infinite loop. */
	for( ;; )
	{
		vTaskDelay( mainDELAY_LOOP_Consume);
		Log_log(LOG_LVL, LV_INFO,(char*)pcTaskName);
		/* Log when there are full queue */
		if( uxQueueMessagesWaiting( xUDPQueue ) == QUEUE_BUZSIZE )
		{
			Log_log(LOG_LVL, LV_ERR, "[UDP] The queue is full!" );
		}

		/* The first parameter is the queue from which data is to be received.  The
		queue is created before the scheduler is started, and therefore before this
		task runs for the first time.

		The second parameter is the buffer into which the received data will be
		placed.  In this case the buffer is simply the address of a variable that
		has the required size to hold the received data.

		the last parameter is the block time � the maximum amount of time that the
		task should remain in the Blocked state to wait for data to be available should
		the queue already be empty. */
		xStatus = xQueueReceive( xUDPQueue, &lReceivedValue, xTicksToWait );


		if( xStatus == pdPASS )
		{
			/* Data was successfully received from the queue, print out the received
			value. */
			while (Udp_sendValue(UDP_SRV_ADDRESS, UDP_SRV_LOCAL_PORT, UDP_SRV_PORT, lReceivedValue)==FALSE)
			{
				Log_log(LOG_LVL, LV_ERR,"[UDP] Cannot send value to UDP server");
			}
			Log_log(LOG_LVL, LV_INFO, "[UDP] Consumer UDP: Sended value to UDP server= %d",lReceivedValue );
		}
		else
		{
			/* We did not receive anything from the queue even after waiting for 100ms.
			This must be an error as the sending tasks are free running and will be
			continuously writing to the queue. */

			Log_log(LOG_LVL, LV_ERR,"[UDP] Don't receive anything from the queue");
		}

	}
}
/*-----------------------------------------------------------*/
void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */
	consoleprint("Error there is too little heap RAM\n");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	consoleprint("Error Stack Overflow\n");
	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}


